from django.apps import AppConfig


class WagtailLocalizeRegionsAdminAppConfig(AppConfig):
    label = 'wagtail_localize_regions_admin'
    name = 'wagtail_localize.admin.regions'
    verbose_name = "Wagtail Localize Regions admin"
