from django.apps import AppConfig


class WagtailLocalizeTestsAppConfig(AppConfig):
    label = 'wagtail_localize_tests'
    name = 'wagtail_localize.tests'
    verbose_name = "Wagtail Localize Tests"
