# Generated by Django 2.1.9 on 2019-06-21 09:16

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('wagtail_localize', '0001_initial'),
        ('wagtailcore', '0040_page_draft_title'),
    ]

    operations = [
        migrations.CreateModel(
            name='TranslationRequest',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField()),
                ('created_by', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='translation_requests_created', to=settings.AUTH_USER_MODEL)),
                ('source_locale', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='wagtail_localize.Locale')),
                ('target_locale', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='wagtail_localize.Locale')),
                ('target_root', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='wagtailcore.Page')),
            ],
        ),
        migrations.CreateModel(
            name='TranslationRequestPage',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_completed', models.BooleanField(default=False)),
                ('completed_revision', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='wagtailcore.PageRevision')),
                ('parent', models.ForeignKey(null=True, on_delete=django.db.models.deletion.PROTECT, related_name='child_pages', to='wagtail_localize_workflow.TranslationRequestPage')),
                ('request', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='pages', to='wagtail_localize_workflow.TranslationRequest')),
                ('source_revision', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='wagtailcore.PageRevision')),
            ],
        ),
    ]
