# Generated by Django 2.1.10 on 2019-08-13 12:30

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('wagtail_localize', '0002_initial_data'),
        ('wagtailcore', '0041_group_collection_permissions_verbose_name_plural'),
    ]

    operations = [
        migrations.CreateModel(
            name='Segment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uuid', models.UUIDField(unique=True)),
                ('text', models.TextField()),
                ('language', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='wagtail_localize.Language')),
            ],
        ),
        migrations.CreateModel(
            name='SegmentPageLocation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('path', models.TextField()),
                ('page_revision', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='wagtailcore.PageRevision')),
                ('segment', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='page_locations', to='wagtail_localize_translation_memory.Segment')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='SegmentTranslation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('text', models.TextField()),
                ('language', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='wagtail_localize.Language')),
                ('translation_of', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='wagtail_localize_translation_memory.Segment')),
            ],
        ),
        migrations.CreateModel(
            name='Template',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uuid', models.UUIDField(unique=True)),
                ('template', models.TextField()),
                ('template_format', models.CharField(max_length=100)),
                ('segment_count', models.PositiveIntegerField()),
            ],
        ),
        migrations.CreateModel(
            name='TemplatePageLocation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('path', models.TextField()),
                ('page_revision', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='wagtailcore.PageRevision')),
                ('template', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='page_locations', to='wagtail_localize_translation_memory.Template')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AlterUniqueTogether(
            name='segmenttranslation',
            unique_together={('language', 'translation_of')},
        ),
    ]
