from django.apps import AppConfig


class WagtailLocalizeWorkflowAppConfig(AppConfig):
    label = 'wagtail_localize_workflow'
    name = 'wagtail_localize.admin.workflow'
    verbose_name = "Wagtail Localize Workflow"
