# Generated by Django 2.1.10 on 2019-08-13 15:33

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wagtail_localize_translation_memory', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='segmentpagelocation',
            name='order',
            field=models.PositiveIntegerField(default=0),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='templatepagelocation',
            name='order',
            field=models.PositiveIntegerField(default=0),
            preserve_default=False,
        ),
    ]
