from django.apps import AppConfig


class WagtailLocalizeSitesAppConfig(AppConfig):
    label = 'wagtail_localize_sites'
    name = 'wagtail_localize.experimental.sites'
    verbose_name = "Wagtail Localize Sites"
