from django.apps import AppConfig


class WagtailLocalizeLanguageSwitchAppConfig(AppConfig):
    label = 'wagtail_localize_language_switch'
    name = 'wagtail_localize.admin.language_switch'
    verbose_name = "Wagtail Localize Language switch"
